<?php

namespace Drupal\Tests\dynamic_linsk\Functional;

use Drupal\Core\Url;
use Drupal\dynamic_links\Routing\DynamicLinksRoutes;
use Drupal\Tests\BrowserTestBase;

/**
 * @coversDefaultClass \Drupal\dynamic_links\Controller\DynamicLinksController
 */
class DynamicLinksTest extends BrowserTestBase {

  /**
   * {@inheritdoc}
   */
  protected static $modules = [
    'dynamic_links_test',
  ];

  /**
   * {@inheritdoc}
   */
  protected $defaultTheme = 'stark';

  /**
   * Tests the redirects.
   *
   * @covers ::redirect
   */
  public function testRedirects() {
    $links = [
      [
        'id' => 'front',
        'variants' => [
          [
            'permissions' => ['dynamic_links_test route_1'],
            'expected' => '/dynamic-links-test/route-1',
            'path_info' => '/dynamic-links-test/route-1',
            'code' => 200,
          ],
          [
            'permissions' => ['dynamic_links_test route_2'],
            'expected' => '/dynamic-links-test/route-2',
            'path_info' => '/dynamic-links-test/route-2',
            'code' => 200,
          ],
          [
            'permissions' => ['dynamic_links_test route_3'],
            'expected' => '/dynamic-links-test/route-3',
            'path_info' => '/dynamic-links-test/route-3',
            'code' => 200,
          ],
          [
            'permissions' => [],
            'expected' => '/dynamic-front',
            'path_info' => FALSE,
            'code' => 403,
          ],
        ],
      ],
      [
        'id' => 'front_as_routes',
        'variants' => [
          [
            'permissions' => ['dynamic_links_test route_1'],
            'expected' => '/dynamic-links-test/route-1',
            'path_info' => '/dynamic-links-test/route-1',
            'code' => 200,
          ],
          [
            'permissions' => ['dynamic_links_test route_2'],
            'expected' => '/dynamic-links-test/route-2',
            'path_info' => '/dynamic-links-test/route-2',
            'code' => 200,
          ],
          [
            'permissions' => ['dynamic_links_test route_3'],
            'expected' => '/dynamic-links-test/route-3',
            'path_info' => '/dynamic-links-test/route-3',
            'code' => 200,
          ],
          [
            'permissions' => [],
            'expected' => '/dynamic-front/as-routes',
            'path_info' => FALSE,
            'code' => 403,
          ],
        ],
      ],
      [
        'id' => 'subrequest_front',
        'variants' => [
          [
            'permissions' => ['dynamic_links_test route_1'],
            'expected' => '/dynamic-front/subrequest',
            'path_info' => '/dynamic-links-test/route-1',
            'code' => 200,
          ],
          [
            'permissions' => ['dynamic_links_test route_2'],
            'expected' => '/dynamic-front/subrequest',
            'path_info' => '/dynamic-links-test/route-2',
            'code' => 200,
          ],
          [
            'permissions' => ['dynamic_links_test route_3'],
            'expected' => '/dynamic-front/subrequest',
            'path_info' => '/dynamic-links-test/route-3',
            'code' => 200,
          ],
          [
            'permissions' => [],
            'expected' => '/dynamic-front/subrequest',
            'path_info' => FALSE,
            'code' => 403,
          ],
        ],
      ],
      [
        'id' => 'subrequest_front_as_routes',
        'variants' => [
          [
            'permissions' => ['dynamic_links_test route_1'],
            'expected' => '/dynamic-front/as-routes/subrequest',
            'path_info' => '/dynamic-links-test/route-1',
            'code' => 200,
          ],
          [
            'permissions' => ['dynamic_links_test route_2'],
            'expected' => '/dynamic-front/as-routes/subrequest',
            'path_info' => '/dynamic-links-test/route-2',
            'code' => 200,
          ],
          [
            'permissions' => ['dynamic_links_test route_3'],
            'expected' => '/dynamic-front/as-routes/subrequest',
            'path_info' => '/dynamic-links-test/route-3',
            'code' => 200,
          ],
          [
            'permissions' => [],
            'expected' => '/dynamic-front/as-routes/subrequest',
            'path_info' => FALSE,
            'code' => 403,
          ],
        ],
      ],
    ];

    foreach ($links as $link) {
      $dynamic_link = Url::fromRoute(DynamicLinksRoutes::ROUTE_PREFIX . $link['id']);
      foreach ($link['variants'] as $data) {
        $user = $this->drupalCreateUser($data['permissions']);
        $this->drupalLogin($user);
        $expected = Url::fromUserInput($data['expected'])->setAbsolute()->toString();
        $this->drupalGet($dynamic_link);
        $session = $this->getSession();
        $this->assertEquals($session->getStatusCode(), $data['code']);
        $this->assertEquals($session->getCurrentUrl(), $expected);
        if ($data['path_info']) {
          $this->xpath('//div[@data-path-info="' . $data['path_info'] . '"]');
        }
        $this->drupalLogout();
      }
    }
  }

}
